@@ -100,6 +100,7 @@ typedef struct {
 } IcsTextMenuEntry;
 
 IcsTextMenuEntry icsTextMenuEntry[ICS_TEXT_MENU_SIZE];
+char dataDir[MSG_SIZ] = DATADIR;
 
 int junk;
 int saveDate;
@@ -711,6 +712,11 @@ ArgDescriptor argDescriptors[] = {
   { "firstDrawDepth", ArgInt, (void *) &appData.drawDepth[0], FALSE, (ArgIniType) 0 },
   { "secondDrawDepth", ArgInt, (void *) &appData.drawDepth[1], FALSE, (ArgIniType) 0 },
   { "memoHeaders", ArgBoolean, (void *) &appData.headers, TRUE, (ArgIniType) FALSE },
+  { "startupMessage", ArgString, (void *) &appData.message, FALSE, (ArgIniType) "" },
+  { "messageSuppress", ArgString, (void *) &appData.suppress, XBOARD, (ArgIniType) "" },
+  { "fen", ArgString, (void *) &appData.fen, FALSE, (ArgIniType) "" },
+  { "men", ArgString, (void *) &appData.men, FALSE, (ArgIniType) "" },
+  { "analysisBell", ArgInt, (void *) &appData.analysisBell, TRUE, (ArgIniType) 0 },
 
 #if ZIPPY
   { "zippyTalk", ArgBoolean, (void *) &appData.zippyTalk, FALSE, (ArgIniType) ZIPPY_TALK },
@@ -929,7 +935,7 @@ ParseSettingsFile(char *name, char **addr)
 	MySearchPath(installDir, buf, fullname); // first look in user's own files
 	f = fopen(fullname, "r");
 	if(f == NULL) {
-	    snprintf(buf, MSG_SIZ, "%s/themes/conf", DATADIR);
+	    snprintf(buf, MSG_SIZ, "%s/themes/conf", dataDir);
 	    MySearchPath(buf, name, fullname); // also look in standard place
 	    f = fopen(fullname, "r");
 	}
@@ -1169,7 +1175,7 @@ ParseArgs(GetFunc get, void *cl)
     case ArgFilename:
       if(argValue[0] == '~' && argValue[1] == '~') {
         char buf[4*MSG_SIZ]; // expand ~~
-        snprintf(buf, 4*MSG_SIZ, "%s%s", DATADIR, argValue+2);
+        snprintf(buf, 4*MSG_SIZ, "%s%s", dataDir, argValue+2);
         ASSIGN(*(char **) ad->argLoc, buf);
         break;
       }
@@ -1410,7 +1416,11 @@ InitAppData(char *lpCmdLine)
      appData.NrRanks > BOARD_RANKS   )
       DisplayFatalError("Recompile with BOARD_RANKS or BOARD_FILES, to support this size", 0, 2);
 
-  if(!*appData.secondChessProgram) { ASSIGN(appData.secondChessProgram, appData.firstChessProgram); } // [HGM] scp defaults to fcp
+  if(!*appData.secondChessProgram) { // [HGM] scp defaults to fcp
+    ASSIGN(appData.secondChessProgram, appData.firstChessProgram);
+    ASSIGN(appData.secondDirectory, appData.firstDirectory);
+    appData.secondIsUCI = appData.firstIsUCI; // copy type too!
+  }
 
   /* [HGM] After parsing the options from the .ini file, and overruling them
    * with options from the command line, we now make an even higher priority
@@ -1626,8 +1636,8 @@ SaveSettings(char* name)
       { char buf[MSG_SIZ];
         snprintf(buf, MSG_SIZ, "%s", *(char**)ad->argLoc);
 #ifdef OSXAPP
-        if(strstr(buf, DATADIR) == buf)
-          snprintf(buf, MSG_SIZ, "~~%s", *(char**)ad->argLoc + strlen(DATADIR));
+        if(strstr(buf, dataDir) == buf)
+          snprintf(buf, MSG_SIZ, "~~%s", *(char**)ad->argLoc + strlen(dataDir));
 #endif
         if (strchr(buf, '\"')) {
           fprintf(f, OPTCHAR "%s" SEPCHAR "'%s'\n", ad->argName, buf);
