@@ -142,6 +142,7 @@ void Wheel P((int dir, int x, int y));
 int StartChildProcess P((char *cmdLine, char *dir, ProcRef *pr));
 void DestroyChildProcess P((ProcRef pr, int/*boolean*/ signal));
 void InterruptChildProcess P((ProcRef pr));
+char *BufferCommandOutput P((char *command, int size));
 void RunCommand P((char *buf));
 
 int OpenTelnet P((char *host, char *port, ProcRef *pr));
@@ -167,6 +168,7 @@ void CmailSigHandlerCallBack P((InputSourceRef isr, VOIDSTAR closure,
 
 extern ProcRef cmailPR;
 extern int shiftKey, controlKey;
+extern char dataDir[], manDir[];
 
 /* in xgamelist.c or winboard.c */
 void GLT_ClearList();
