@@ -269,6 +269,45 @@ SetWidgetFont (GtkWidget *w, char **s)
     gtk_widget_modify_font(w, pfd);
 }
 
+void
+ApplyFont (Option *opt, char *font)
+{
+    GtkWidget *w = NULL;
+    if(!font && opt->font) font = *opt->font;
+    if(!font) return;
+    switch(opt->type) {
+      case ListBox:
+      case Label:   w = opt->handle; break;
+      case Button:  if(opt->handle) w = gtk_bin_get_child(GTK_BIN(opt->handle)); break;
+      case TextBox: w = (GtkWidget *) opt->textValue; if(!w) w = opt->handle; break;
+      default: ;
+    }
+    if(w && font) SetWidgetFont(w, &font);
+}
+
+GtkWidget *fbutton;
+
+void
+FontCallback (GtkWidget *widget, gpointer gdata)
+{
+    Option *opt = (Option *) gdata;
+    gchar *p = (char *) gtk_font_button_get_font_name(GTK_FONT_BUTTON(fbutton));
+    SetWidgetText(opt, p, TransientDlg);
+    ApplyFont(opt, p);
+}
+
+void
+ColorCallback (GtkWidget *widget, gpointer gdata)
+{
+    Option *opt = (Option *) gdata;
+    GdkColor rgba;
+    char buf[MSG_SIZ]; 
+    gtk_color_button_get_color(GTK_COLOR_BUTTON(widget), &rgba);
+    snprintf(buf, MSG_SIZ, "#%02x%02x%02x", rgba.red>>8, rgba.green>>8, rgba.blue>>8);
+    gtk_widget_modify_bg ( GTK_WIDGET(opt[1].handle), GTK_STATE_NORMAL, &rgba );
+    SetWidgetText(opt, buf, TransientDlg);
+}
+
 void
 SetListBoxItem (GtkListStore *store, int n, char *msg)
 {
@@ -599,6 +638,11 @@ AppendColorized (Option *opt, char *s, int count)
     static GtkTextIter end;
     static GtkTextTag *fgTags[8], *bgTags[8], *font, *bold, *normal, *attr = NULL;
 
+    if(!s) {
+	font = NULL;
+	return;
+    }
+
     if(!font) {
 	font = gtk_text_buffer_create_tag(opt->handle, NULL, "font", appData.icsFont, NULL);
 	gtk_widget_modify_base(GTK_WIDGET(opt->textValue), GTK_STATE_NORMAL, &backgroundColor);
@@ -912,6 +956,21 @@ ColorChanged (Widget w, XtPointer data, XEvent *event, Boolean *b)
 }
 #endif
 
+static void
+ExposeDraw (Option *graph, GdkEventExpose *eevent)
+{
+    int w = eevent->area.width;
+    cairo_t *cr;
+    if(eevent->area.x + w > graph->max) w--; // cut off fudge pixel
+    cr = gdk_cairo_create(((GtkWidget *) (graph->handle))->window);
+    cairo_set_source_surface(cr, (cairo_surface_t *) graph->choice, 0, 0);
+//cairo_set_source_rgb(cr, 1, 0, 0);
+    cairo_set_antialias(cr, CAIRO_ANTIALIAS_NONE);
+    cairo_rectangle(cr, eevent->area.x, eevent->area.y, w, eevent->area.height);
+    cairo_fill(cr);
+    cairo_destroy(cr);
+}
+
 static void
 GraphEventProc(GtkWidget *widget, GdkEvent *event, gpointer gdata)
 {   // handle expose and mouse events on Graph widget
@@ -924,7 +983,6 @@ GraphEventProc(GtkWidget *widget, GdkEvent *event, gpointer gdata)
     GdkEventMotion *mevent = (GdkEventMotion *) event;
     GdkEventScroll *sevent = (GdkEventScroll *) event;
     GtkAllocation a;
-    cairo_t *cr;
 
 //    if (!XtIsRealized(widget)) return;
 
@@ -957,26 +1015,11 @@ GraphEventProc(GtkWidget *widget, GdkEvent *event, gpointer gdata)
 			 // to give drawing routines opportunity to use it before first expose event
 			 // (which are only processed when main gets to the event loop, so after all init!)
 			 // so only change when size is no longer good
-		cairo_t *cr;
-		if(graph->choice) cairo_surface_destroy((cairo_surface_t *) graph->choice);
-		graph->choice = (char**) cairo_image_surface_create (CAIRO_FORMAT_ARGB32, w, h);
-		// paint white, to prevent weirdness when people maximize window and drag pieces over space next to board
-		cr = cairo_create ((cairo_surface_t *) graph->choice);
-		cairo_rectangle (cr, 0, 0, w, h);
-		cairo_set_source_rgba(cr, 1.0, 1.0, 1.0, 1.0);
-		cairo_fill(cr);
-		cairo_destroy (cr);
+//		NewCanvas(graph);
+		graph->min |= REPLACE; // defer making new canvas
 		break;
 	    }
-	    w = eevent->area.width;
-	    if(eevent->area.x + w > graph->max) w--; // cut off fudge pixel
-	    cr = gdk_cairo_create(((GtkWidget *) (graph->handle))->window);
-	    cairo_set_source_surface(cr, (cairo_surface_t *) graph->choice, 0, 0);
-//cairo_set_source_rgb(cr, 1, 0, 0);
-	    cairo_set_antialias(cr, CAIRO_ANTIALIAS_NONE);
-	    cairo_rectangle(cr, eevent->area.x, eevent->area.y, w, eevent->area.height);
-	    cairo_fill(cr);
-	    cairo_destroy(cr);
+	    ExposeDraw(graph, eevent);
 	default:
 	    return;
 	case GDK_SCROLL:
@@ -1019,7 +1062,7 @@ GraphExpose (Option *opt, int x, int y, int w, int h)
   GdkEventExpose e;
   if(!opt->handle) return;
   e.area.x = x; e.area.y = y; e.area.width = w; e.area.height = h; e.count = -1; e.type = GDK_EXPOSE; // count = -1: kludge to suppress sizing
-  GraphEventProc(opt->handle, (GdkEvent *) &e, (gpointer) opt); // fake expose event
+  ExposeDraw(opt, &e); // fake expose event
 }
 
 void GenericCallback(GtkWidget *widget, gpointer gdata)
@@ -1368,7 +1411,7 @@ if(appData.debugMode) printf("n=%d, h=%d, w=%d\n",n,height,width);
 	  case FileName:
      	  case PathName:
           tBox:
-            label = gtk_label_new(option[i].name);
+            label = gtk_label_new(_(option[i].name));
             /* Left Justify */
             gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
 
@@ -1452,7 +1495,7 @@ if(appData.debugMode) printf("n=%d, h=%d, w=%d\n",n,height,width);
             }
             else if (option[i].type == FileName || option[i].type == PathName) {
                 gtk_table_attach(GTK_TABLE(table), entry, left+1, left+2, top, top+1, GTK_FILL | GTK_EXPAND, GTK_FILL, 2, 1);
-                button = gtk_button_new_with_label ("Browse");
+                button = gtk_button_new_with_label (_("Browse"));
                 gtk_table_attach(GTK_TABLE(table), button, left+2, left+r, top, top+1, GTK_FILL, GTK_FILL, 2, 1); // Browse button does not expand
                 g_signal_connect (button, "clicked", G_CALLBACK (BrowseGTK), (gpointer)(intptr_t) i);
                 option[i].handle = (void*)entry;
@@ -1463,7 +1506,7 @@ if(appData.debugMode) printf("n=%d, h=%d, w=%d\n",n,height,width);
             }
             break;
           case CheckBox:
-            checkbutton = gtk_check_button_new_with_label(option[i].name);
+            checkbutton = gtk_check_button_new_with_label(_(option[i].name));
 	    g_signal_connect(checkbutton, "button-press-event", G_CALLBACK (HelpEvent), (gpointer) option[i].name );
             if(!currentCps) option[i].value = *(Boolean*)option[i].target;
             gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton), option[i].value);
@@ -1476,7 +1519,7 @@ if(appData.debugMode) printf("n=%d, h=%d, w=%d\n",n,height,width);
             Pack(hbox, table, label, left, left+2, top, 0);
             break;
 	  case Label:
-            option[i].handle = (void *) (label = gtk_label_new(option[i].name));
+            option[i].handle = (void *) (label = gtk_label_new(_(option[i].name)));
             /* Left Justify */
             gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
             SetWidgetFont(label, option[i].font);
@@ -1486,7 +1529,7 @@ if(appData.debugMode) printf("n=%d, h=%d, w=%d\n",n,height,width);
 		label = frame;
 	    }
             gtk_widget_set_size_request(label, option[i].max ? option[i].max : -1, -1);
-	    if(option[i].target || dlgNr != ErrorDlg && option[i].name && !strchr(option[i].name, '\n')) { // allow user to specify event handler for button presses
+	    if(option[i].target || dlgNr != ErrorDlg && option[i].name) { // allow user to specify event handler for button presses
 		button = gtk_event_box_new();
                 gtk_container_add(GTK_CONTAINER(button), label);
 		label = button;
@@ -1500,7 +1543,22 @@ if(appData.debugMode) printf("n=%d, h=%d, w=%d\n",n,height,width);
 	    break;
           case SaveButton:
           case Button:
-            button = gtk_button_new_with_label (option[i].name);
+            if(!strcmp(option[i].name, "fontsel")) {
+                option[i].handle = (void *) (fbutton = gtk_font_button_new());
+                Pack(hbox, table, fbutton, left, left+r, top, 0);
+                break;
+            }
+            if(!strcmp(option[i].name, "R") || !strcmp(option[i].name, "G") || !strcmp(option[i].name, "B")) {
+                break;
+            } else
+            if(!strcmp(option[i].name, "D")) {
+                GdkColor color;
+                char *name;
+		GetWidgetText(&option[i-5], &name);
+                gdk_color_parse(name, &color);
+                option[i].handle = (void *) (button = gtk_color_button_new_with_color(&color));
+            } else
+            button = gtk_button_new_with_label (_(option[i].name));
             SetWidgetFont(gtk_bin_get_child(GTK_BIN(button)), option[i].font);
 
             /* set button color on view board dialog */
@@ -1524,12 +1582,18 @@ if(appData.debugMode) printf("n=%d, h=%d, w=%d\n",n,height,width);
             }
 
             Pack(hbox, table, button, left, left+1, top, 0);
+            if(!strcmp(option[i].name, "D")) // color button
+            g_signal_connect (button, "color-set", G_CALLBACK (ColorCallback), (gpointer) &option[i-5]);
+            else
+            if(option[i].value == 666 && !strcmp(option[i].name, "*")) // font-assignment buttons
+            g_signal_connect (button, "clicked", G_CALLBACK (FontCallback), (gpointer) &option[i-5]);
+            else
             g_signal_connect (button, "clicked", G_CALLBACK (GenericCallback), (gpointer)(intptr_t) i + (dlgNr<<16));
 	    g_signal_connect(button, "button-press-event", G_CALLBACK (HelpEvent), (gpointer) option[i].name );
             option[i].handle = (void*)button;
             break;
 	  case ComboBox:
-            label = gtk_label_new(option[i].name);
+            label = gtk_label_new(_(option[i].name));
             /* Left Justify */
             gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
 	    button = gtk_event_box_new();
