@@ -288,6 +288,11 @@ AppendColorized (Option *opt, char *message, int count)
   AppendText(opt, message);
 }
 
+void
+ApplyFont (Option *opt, char *font)
+{ // dummy
+}
+
 void
 Show (Option *opt, int hide)
 {
